/**
 * A component that renders a popover for a selected/highlighted text.
 *
 * @remarks
 * The `react-highlight-popover` defines the {@link HighlightableTextAreaProps} interface and {@link HighlightableTextArea} function.
 *
 * @packageDocumentation
 */
import * as React from 'react';
import './styles.scss';
declare type popoverItemCallback = (
/**
 * The text that is selected/highlighted.
 */
HighlightedText: string, 
/**
 * Use this to change the state of the popover.
 */
setPopoverState: React.Dispatch<React.SetStateAction<boolean>>) => React.ReactNode;
declare type onHighlightTextCallback = (event: MouseEvent) => void;
/**
 * Defines the props for {@link HighlightableTextArea}
 */
export interface HighlightableTextAreaProps {
    /**
     * The item to be rendered as popover when user has selected/highlighted text.
     * @example
     * Here is how it can be implemented:
     *
     * ```js
     * <HighlightableTextArea
     *    popoverItem={(HighlightedText, setPopoverState) => {
     *      return (
     *        <div onClick={() => setPopoverState(false)}>
     *          <p>{HighlightedText}</p>
     *        </div>
     *    )}}
     * >
     * </HighlightableTextArea>
     * ```
     * @param HighlightedText - The text that is selected/highlighted.
     * @param setPopoverState - Set to `false` to hide the popover.
     * @returns A react node.
     */
    popoverItem?: popoverItemCallback;
    /**
     * This event is fired after the popover renders.
     * It is a callback function that takes `event` as an argument.
     * @eventProperty
     */
    onHighlightText?: onHighlightTextCallback;
    /**
     * Positions the popover along the x-axis of the selected text.
     * @defaultValue 0
     */
    xOffset?: number;
    /**
     * Positions the popover along the y-axis of the selected text.
     * @defaultValue 10
     */
    yOffset?: number;
    /**
     * The `zIndex` property sets the z-order of the popover and its descendants or flex items.
     * Overlapping elements with a larger z-index cover those with a smaller one.
     * @defaultValue 10
     */
    zIndex?: number;
    children: React.ReactNode;
}
/**
 * Defines the area in which selected/highlighted text will render a popover.
 * @public ❤️ from Nigeria
 */
export declare const HighlightableTextArea: ({ popoverItem, onHighlightText, xOffset, yOffset, zIndex, children }: HighlightableTextAreaProps) => JSX.Element;
export {};
